%fr = d2mlf(f,x,y,q) computes the second order directional derivative
%of function f along vectors x and y at the point q.
%f is a dual function D^m --> D^n (D^(pm) --> D^(pn))
%x, y and q are a real (or complex) arrays of the same dimensions
%x = [x11, ... ,x1m], alternatively
%x = [x11, ... ,x1m; x21, ... ,x2m; ... ;xp1, ... ,xpm]
%For convenience, x, y and q are assumed row vectors of dimension m, or
%as matrices of dimensions [p,m]. 

function fr = d2mlf(f,x,y,q)
    fr = 0.5.*(d21mlf(f,x+y,q) - d21mlf(f,x,q) - d21mlf(f,y,q));
end

function fr = d21mlf(f,v,q)
    eps1 =  dual2(0,1,0);
    fqd = f(q + eps1.*v);
    fr = fqd.f2;
end
