%fr = accel_d2(f,g,t) computes the acceleration: d2/dt^2 (f(g(t))) evaluated 
%at t.
%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:D^m --> D^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:D -->D^m
%the position vector as function of t is r = f(g(t))
%
%fr = accel_d2(f,q2p,q1p,q) computes the acceleration vector at the point 
%q = [q1,q2,...,qm], for given vectors q1p=dq/dt, q2p = dq1p/dt
%if f deppends explicitly on time, use qm=t
function fr = accel_d2(varargin)
    if (nargin == 3)  
        fr = accelfL(varargin{:});
    elseif(nargin == 4)
        fr = accelvL(varargin{:});
    else
        error('wrong number of arguments in accel function');       
    end 
end

%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%fr = velf(f,g,t) computes the velocity: d/dt (f(g(t))) evaluated at t
function fr = accelfL(f,g,t)    
    td = dual2(t,1,0);
    frd = f(g(td));
    fr = frd.f2;  
end

%computes the acceleration vector at the point q = [q1,q2,...,qm]
%for given vectors q1p=dq/dt, q2p = dq1p/dt
%if f deppends explicitly on time, use qm=t
function fr = accelvL(f,q2p,q1p,q)
fr = d1mlf(f,q2p,q) + d2mlf(f,q1p,q1p,q);
end



