%fr = jerk_d3(f,g,t) computes the jerk: d3/dt^3 (f(g(t))) evaluated at t
%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%
%fr = jerk_d3(f,q3p,q2p,q1p,q) computes the jerk vector at the point 
%q = [q1,q2,...,qm], for given vectors q1p=dq/dt, q2p = dq1p/dt, 
%q3p = dq2p/dt
%if f deppends explicitly on time, use qm=t
function fr = jerk_d3(varargin)
    if (nargin == 3)  
        fr = jerkfL(varargin{:});
    elseif(nargin == 5)
        fr = jerkvL(varargin{:});
    else
        error('wrong number of arguments in jerk function');       
    end 
end

%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%fr = velf(f,g,t) computes the velocity: d/dt (f(g(t))) evaluated at t
function fr = jerkfL(f,g,t)   
    td = dual3(t,1,0,0);
    frd = f(g(td));
    fr = frd.f3;    
end

%computes the jerk vector at the point q = [q1,q2,...,qm]
%for given vectors q1p=dq/dt, q2p = dq1p/dt, q3p = dq2p/dt
%if f deppends explicitly on time, use qm=t
function fr = jerkvL(f,q3p,q2p,q1p,q)
fr = d3mlf(f,q1p,q1p,q1p,q) + 3.*d2mlf(f,q2p,q1p,q) + ...
    d1mlf(f,q3p,q);
end



