%fr = jounce_snap_d4(f,g,t) computes the jerk: d4/dt^4 (f(g(t))) evaluated at 
%t. 
%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%
%fr = jounce_snap_d4(f,q4p,q3p,q2p,q1p,q) computes the jounce/snap vector at 
%the point q = [q1,q2,...,qm], for given vectors q1p=dq/dt, q2p = dq1p/dt, 
%q3p = dq2p/dt, q4p = dq3p/dt
%if f deppends explicitly on time, use qm=t
function fr = jounce_snap_d4(varargin)
    if (nargin == 3)  
        fr = jounce_snapfL(varargin{:});
    elseif(nargin == 6)
        fr = jounce_snapvL(varargin{:});
    else
        error('wrong number of arguments in jounce_snap function');       
    end 
end

%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%fr = jounce_snapfL(f,g,t) computes the jounce/snap: d4/dt^4 (f(g(t))) 
%evaluated at t
function fr = jounce_snapfL(f,g,t)     
    td = dual4(t,1,0,0,0);
    frd = f(g(td));
    fr = frd.f4;     
end

%computes the jounce/snap vector at the point q=[q1,q2,...,qm]
%for given vectors q1p=dq/dt, q2p=dq1p/dt, q3p=dq2p/dt, q4p=dq3p/dt
%if f deppends explicitly on time, use qm=t
function fr = jounce_snapvL(f,q4p,q3p,q2p,q1p,q)
fr = d4mlf(f,q1p,q1p,q1p,q1p,q) + 6.*d3mlf(f,q1p,q1p,q2p,q) + ...
    4.*d2mlf(f,q1p,q3p,q) + 3.*d2mlf(f,q2p,q2p,q) + d1mlf(f,q4p,q);
end

