%fr = vel_d1(a1,a2,a3) computes the velocity vector of the function 
%a1(q1,q2,...qm)
%a2 can be a function: a2(t) = [q1(t),q2(t),...,qm(t)]; is the vector of 
%generalized coordinates as function of t.
%if a2 is a function, then a3 must be a number (the time)
%a2 can also be a vector q1p = d/dt([q1,q2,...qm]), if this is the case,
%then a3 must be also a vector q = [q1,q2,...qm]
%if f deppends explicitly on time, use qm=t
function fr = vel_d1(a1,a2,a3)
    if isa(a2, 'function_handle')
        fr = velfL(a1,a2,a3);
    else
        fr = velvL(a1,a2,a3);
    end 
end

%f(q1,q2,...qm) is the position vector as function of the generalized 
%coordinates, including t. f:R^m --> R^n
%g(t) = [q1(t),q2(t),...,qm(t)] is the vector of generalized coordinates as
%function of t. g:R -->R^m
%the position vector as function of t is r = f(g(t))
%if f deppends explicitly on time, use qm(t) = t
%fr = velf(f,g,t) computes the velocity: d/dt (f(g(t))) evaluated at t
function fr = velfL(f,g,t)   
    td = dual1(t,1);
    frd = f(g(td));
    fr = frd.f1;  
end


%computes the velocity vector at the point q = [q1,q2,...,qm]
%for a given vector q1p = [dq1/dt, dq2/dt, ..., dqm/dt]
%if f deppends explicitly on time, use qm=t
function fr = velvL(f,q1p,q)
fr = d1mlf(f,q1p,q);
end
