addpath('01DualNumbersF;02DMLF;03KinematicQuantities_dmlf')


%example RCR robot 
fprintf('\n')
disp('exaple RCR')
th0=pi/2; phi0=0; s0=2; beta0=0;
th1=1;    phi1=5; s1=1; beta1=1; 
th2=1;    phi2=0; s2=2; beta2=1; 
th3=1;    phi3=2; s3=3; beta3=4;
th4=4;    phi4=5; s4=6; beta4=7;

q0p=[th0,phi0,s0,beta0];
q1p=[th1,phi1,s1,beta1];
q2p=[th2,phi2,s2,beta2];
q3p=[th3,phi3,s3,beta3];
q4p=[th4,phi4,s4,beta4];

disp('velocity, acceleration, jerk and jounce/snap')
tic()
v1 =  vel_d1(@fE3,q1p,q0p);
a1 = accel_d2(@fE3,q2p,q1p,q0p);
j1 = jerk_d3(@fE3,q3p,q2p,q1p,q0p);
js1=jounce_snap_d4(@fE3,q4p,q3p,q2p,q1p,q0p);

disp([v1;a1;j1;js1])
toc()

  
%example 
%q=[th,phi,s,betha]
%position vector of the RCR robot manipulator
function fr = fE3(q)
n2cq = num2cell(q,1);
[th,phi,s,beta] = n2cq{:};
BC  = 3;
CD  = 2;
T1 = HTM(th,[0,0,1],[0,0,0]);
T2 = HTM(phi,[1,0,0],[s,0,0]);
T3 = HTM(beta,[0,0,1],[0,0,0]);
T4 = HTM(0,[0,0,1],[0,0,BC]);
T5 = HTM(0,[1,0,0],[CD,0,0]);
fr4 = T1*T2*T3*T4*T5;
fr  = fr4(1:3,4);
fr = fr.';
end 

%Homogeneous transformation matrix
function fr = HTM(th,eje,Dr)
fr(1:3,1:3) = rot_mat_aux(th,eje);
fr(1:3,4) = Dr;
fr(4,:) = [0,0,0,1];
end

%rotation matrix
function fr = rot_mat_aux(th,eje)     
%making 'eje' a unit vector
ejeu = eje./sqrt(sum(eje.*eje));
      
n1 = ejeu(1); n2 = ejeu(2); n3 = ejeu(3);
      
fr(1,:) = [1 + (n2.^2 + n3.^2).*(cos(th) - 1), n1.*n2 - n1.*n2.* ...
    cos(th) - n3.*sin(th), n1.*n3 - n1.*n3.*cos(th) + n2.*sin(th)];

fr(2,:) = [n1.*n2 - n1.*n2.*cos(th) + n3.*sin(th), 1 + (n1.^2 + ...
    n3.^2).*(cos(th) - 1), n2.*n3 - n2.*n3.*cos(th) - n1.*sin(th)];

fr(3,:) = [n1.*n3 - n1.*n3.*cos(th) - n2.*sin(th), n2.*n3 - n2.*n3.* ...
    cos(th) + n1.*sin(th), 1 + (n1.^2 + n2.^2).*(cos(th) - 1)];
end

